@@ -31,6 +31,7 @@
 #include <winsock2.h>	// needed since libraw 0.16
 #endif
 
+#include <QWidget>
 #include <QDialog>
 #include <QLabel>
 #include <QRadioButton>
@@ -51,6 +52,9 @@
 #include <QHeaderView>
 #include <QTreeView>
 #include <QMimeData>
+#include <QStringListModel>
+#include <QListView>
+#include <QDialogButtonBox>
 
 #include <QPrintPreviewWidget>
 #include <QPageSetupDialog>
@@ -92,38 +96,6 @@ private:
 	bool selectOnMousePressEvent; 
 };
 
-class DkMessageBox : public QDialog {
-	Q_OBJECT
-
-public:
-	DkMessageBox(QMessageBox::Icon icon, 
-		const QString& title, 
-		const QString& text, 
-		QMessageBox::StandardButtons buttons = QMessageBox::NoButton,
-		QWidget* parent = 0, 
-		Qt::WindowFlags f = Qt::Dialog);
-	DkMessageBox(QWidget* parent = 0);
-
-	~DkMessageBox();
-
-	virtual void setVisible(bool visible);
-
-public slots:
-	void buttonClicked(QAbstractButton* button);
-	int exec();
-
-protected:
-	
-	QLabel* iconLabel;
-	QLabel* textLabel;
-	QMessageBox::Icon icon;
-	QDialogButtonBox* buttonBox;
-	QCheckBox* showAgain;
-
-	void createLayout(const QMessageBox::Icon& userIcon, const QString& userText, QMessageBox::StandardButtons buttons);
-	void updateSize();
-};
-
 class DkSelectAllDoubleSpinBox : public QDoubleSpinBox {
 	public:
 		DkSelectAllDoubleSpinBox(QWidget* parent = 0) : QDoubleSpinBox(parent) {
@@ -342,7 +314,7 @@ class DkResizeDialog : public QDialog {
 
 public:
 	DkResizeDialog(QWidget* parent = 0, Qt::WindowFlags flags = 0);
-	~DkResizeDialog();
+	~DkResizeDialog() {};
 
 	enum{ipl_nearest, ipl_area, ipl_linear, ipl_cubic, ipl_lanczos, ipl_end};
 	enum{size_pixel, size_percent, size_end};
@@ -351,7 +323,7 @@ public:
 
 	void setImage(QImage img) {
 		this->img = img;
-		initBoxes();
+		initBoxes(true);
 		updateSnippets();
 		drawPreview();
 		wPixelEdit->selectAll();
@@ -406,6 +378,9 @@ protected slots:
 		QDialog::setVisible(visible);
 	}
 
+public slots:
+	virtual void accept();
+
 protected:
 	int leftSpacing;
 	int margin;
@@ -431,6 +406,7 @@ protected:
 	QDoubleSpinBox* resolutionEdit;
 	QComboBox* resUnitBox;
 	QCheckBox* resampleCheck;
+	QCheckBox* gammaCorrection;
 	QComboBox* resampleBox;
 
 	float exifDpi;
@@ -438,7 +414,7 @@ protected:
 	QVector<float> resFactor;
 
 	void init();
-	void initBoxes();
+	void initBoxes(bool updateSettings = false);
 	void createLayout();
 	void updateSnippets();
 	void updateHeight();
@@ -509,7 +485,6 @@ public:
 	TreeItem* find(const QVariant& value, int column);
 	void setParent(TreeItem* parent);
 
-
 private:
 	QVector<TreeItem*> childItems;
 	QVector<QVariant> itemData;
@@ -676,7 +651,7 @@ public:
 	};
 
 	DkPrintPreviewDialog(QImage img, float dpi, QPrinter* printer = 0, QWidget* parent = 0, Qt::WindowFlags flags = 0);
-
+	void setImage(QImage img, float dpi);
 	void init();
 
 public slots:
@@ -703,6 +678,7 @@ private slots:
 
 private:
 	void setFitting(bool on);
+	void scaleImage();
 	bool isFitting() {
 		return (fitGroup->isExclusive() && (fitWidthAction->isChecked() || fitPageAction->isChecked()));
 	};
@@ -819,6 +795,43 @@ protected:
 	};
 };
 #ifdef WITH_OPENCV
+class DkUnsharpDialog : public QDialog {
+	Q_OBJECT
+
+public:
+	DkUnsharpDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);
+	QImage getImage();
+
+public slots:
+	void on_sigmaSlider_valueChanged(int i);
+	void on_amountSlider_valueChanged(int i);
+	void setFile(const QFileInfo& file);
+	void setImage(const QImage& img);
+	void computePreview();
+	void reject();
+	QImage computeUnsharp(const QImage img, int sigma, int amount);
+	void unsharpFinished();
+
+signals:
+	void updateImage(QImage img);
+
+protected:
+	void createLayout();
+	void dropEvent(QDropEvent *event);
+	void dragEnterEvent(QDragEnterEvent *event);
+
+	DkBaseViewPort* viewport;
+	QLabel* preview;
+	QDialogButtonBox* buttons;
+	QFutureWatcher<QImage> unsharpWatcher;
+
+	DkSlider* sigmaSlider;
+	DkSlider* amountSlider;
+
+	bool processing;
+	QImage img;
+};
+
 class DkMosaicDialog : public QDialog {
 	Q_OBJECT
 
@@ -931,5 +944,24 @@ protected:
 	QCheckBox* cbForceSave;
 };
 
+class DkWelcomeDialog : public QDialog {
+	Q_OBJECT
+
+public:
+	DkWelcomeDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);
+
+	bool isLanguageChanged();
+
+public slots:
+	virtual void accept();
+
+protected:
+	void createLayout();
+	
+	QComboBox* languageCombo;
+	QCheckBox* registerFilesCheckBox;
+	QStringList languages;
+	bool languageChanged;
+};
 
 }
