@@ -61,6 +61,7 @@
 	#pragma warning(disable: 4996)
 #endif
 
+
 #ifdef DISABLE_LANCZOS // opencv 2.1.0 is used, does not have opencv2 includes
 #include "opencv/cv.h"
 #else
@@ -91,6 +92,7 @@ namespace nmc {
 
 class DkTcpMenu;
 class DkCompressDialog;
+class DkSettingsDialog;
 class DkTifDialog;
 class DkOpacityDialog;
 class DkResizeDialog;
@@ -99,13 +101,26 @@ class DkForceThumbDialog;
 class DkTrainDialog;
 class DkExplorer;
 class DkExportTiffDialog;
-class DkMosaicDialog;
 class DkImageManipulationDialog;
 class DkUpdater;
+class DkTranslationUpdater;
 class DkLocalManagerThread;
 class DkLanManagerThread;
+class DkRCManagerThread;
 class DkTransferToolBar;
+class DkPluginManager;
 class DkAppManager;
+class DkImageContainerT;	// TODO: add include to suppress warning C4150
+class DkThumbsSaver;
+class DkPrintPreviewDialog;
+class FileDownloader;
+
+#ifdef WITH_UPNP
+class DkUpnpControlPoint;
+class DkUpnpDeviceHost;
+class DkUpnpRendererDeviceHost;
+#endif // WITH_UPNP
+
 
 
 // keyboard shortcuts
@@ -165,6 +180,7 @@ enum {
 	shortcut_tp_pattern		= Qt::Key_B,
 	shortcut_anti_aliasing	= Qt::Key_A,
 	shortcut_lock_window	= Qt::CTRL + Qt::SHIFT + Qt::ALT + Qt::Key_B,
+	shortcut_recent_files	= Qt::CTRL + Qt::Key_R,
 	//shortcut_play			= Qt::Key_Space,
 
 	// edit
@@ -176,6 +192,7 @@ enum {
 	shortcut_delete_silent	= Qt::SHIFT + Qt::Key_Delete,
 	shortcut_crop			= Qt::Key_C,
 	shortcut_copy_buffer	= Qt::CTRL + Qt::SHIFT + Qt::Key_C,
+	shortcut_copy_color		= Qt::CTRL + Qt::ALT + Qt::Key_C,
 	shortcut_auto_adjust	= Qt::CTRL + Qt::SHIFT + Qt::Key_L,
 	shortcut_norm_image		= Qt::CTRL + Qt::SHIFT + Qt::Key_N,
 
@@ -209,6 +226,7 @@ enum fileActions {
 	menu_file_goto,
 	menu_file_find,
 	menu_file_recursive,
+	menu_file_show_recent,
 	menu_file_print,
 	menu_file_reload,
 	menu_file_next,
@@ -226,9 +244,9 @@ enum sortActions {
 	menu_sort_filename,
 	menu_sort_date_created,
 	menu_sort_date_modified,
+	menu_sort_random,
 	menu_sort_ascending,
 	menu_sort_descending,
-	menu_sort_random,
 
 	menu_sort_end,
 };
@@ -239,6 +257,7 @@ enum editActions {
 	menu_edit_rotate_180,
 	menu_edit_copy,
 	menu_edit_copy_buffer,
+	menu_edit_copy_color,
 	menu_edit_paste,
 	menu_edit_shortcuts,
 	menu_edit_preferences,
@@ -250,6 +269,7 @@ enum editActions {
 	menu_edit_invert,
 	menu_edit_norm,
 	menu_edit_auto_adjust,
+	menu_edit_unsharp,
 	menu_edit_wallpaper,
 
 	menu_edit_end,	// nothing beyond this point
@@ -312,10 +332,21 @@ enum syncActions {
 	menu_sync_pos,
 	menu_sync_arrange,
 	menu_sync_connect_all,
+	menu_sync_all_actions,
+	menu_sync_start_upnp,
+
+	menu_sync_remote_control,
+	menu_sync_remote_display,
 
 	menu_sync_end,	// nothing beyond this point
 };
 
+enum pluginsActions {
+	menu_plugin_manager,
+	
+	menu_plugins_end,	// nothing beyond this point
+};
+
 enum lanSyncActions {
 	menu_lan_server,
 	menu_lan_image,
@@ -325,6 +356,7 @@ enum lanSyncActions {
 
 enum helpActions {
 	menu_help_update,
+	menu_help_update_translation,
 	menu_help_bug,
 	menu_help_feature,
 	menu_help_documentation,
@@ -434,18 +466,22 @@ public:
 	QVector<QAction* > getPanelActions();
 	QVector<QAction* > getViewActions();
 	QVector<QAction* > getSyncActions();
-	void loadFile(const QFileInfo& file, bool silent = false);
+	void loadFile(const QFileInfo& file);
 
 	static void updateAll();
 
 	bool saveSettings;
 
+	QString getCurrRunningPlugin() {return currRunningPlugin;};
+
 signals:
 	void sendTitleSignal(QString newTitle);
 	void sendPositionSignal(QRect newRect, bool overlaid);
 	void sendArrangeSignal(bool overlaid);
 	void synchronizeWithSignal(quint16);
+	void stopSynchronizeWithSignal();
 	void synchronizeWithServerPortSignal(quint16);
+	void synchronizeRemoteControl(quint16);
 	void closeSignal();
 	void saveTempFileSignal(QImage img);
 	void sendQuitLocalClientsSignal();
@@ -457,6 +493,7 @@ public slots:
 	void openKeyboardShortcuts();
 	void openSettings();
 	void showExplorer(bool show);
+	void showRecentFiles(bool show = true);
 	void openDir();
 	void openFile();
 	void renameFile();
@@ -489,8 +526,10 @@ public slots:
 	void openDocumentation();
 	void bugReport();
 	void featureRequest();
-	void errorDialog(QString msg, QString title = "Error");
+	//void errorDialog(QString msg, QString title = "Error");
+	void errorDialog(const QString& msg);
 	void loadRecursion();
+	void setWindowTitle(QSharedPointer<DkImageContainerT> imgC);
 	void setWindowTitle(QFileInfo file, QSize size = QSize(), bool edited = false, QString attr = QString());
 	void showOpacityDialog();
 	void opacityUp();
@@ -507,11 +546,13 @@ public slots:
 	void showStatusMessage(QString msg, int which = status_pixel_info);
 	void copyImage();
 	void copyImageBuffer();
+	void copyPixelColorValue();
 	void pasteImage();
 	void flipImageHorizontal();
 	void flipImageVertical();
 	void normalizeImage();
 	void autoAdjustImage();
+	void unsharpMask();
 	void invertImage();
 	virtual void settingsChanged();
 	void showUpdaterMessage(QString msg, QString title);
@@ -519,6 +560,7 @@ public slots:
 	void performUpdate();
 	void updateProgress(qint64 received, qint64 total);
 	void startSetup(QString);
+	void updateTranslations();
 	virtual void enableNoImageActions(bool enable = true);
 	void checkForUpdate();
 	void setFrameless(bool frameless);
@@ -526,12 +568,21 @@ public slots:
 	void setRecursiveScan(bool recursive);
 	void setContrast(bool contrast);
 	void enableMovieActions(bool enable);
+	void runLoadedPlugin();
+	void openPluginManager();
+	void initPluginManager();
+	void runPluginFromShortcut();
+	void closePlugin(bool askForSaving, bool alreadySaving);
+	void applyPluginChanges(bool askForSaving, bool alreadySaving);
 	void clearFileHistory();
 	void clearFolderHistory();
+	void downloadFile(const QUrl& url);
+	void fileDownloaded();
 	//void shareFacebook();
 
 	// batch actions
 	void computeThumbsBatch();
+	void onWindowLoaded();
 
 protected:
 	
@@ -561,6 +612,8 @@ protected:
 	bool gestureEvent(QGestureEvent *event);
 
 	void assignCustomShortcuts(QVector<QAction*> actions);
+	void assignCustomPluginShortcuts();
+	void savePluginActions(QVector<QAction *> actions);
 
 	bool otherKeyPressed;
 	QPoint posGrabKey;
@@ -568,6 +621,8 @@ protected:
 
 	// vars
 	QWidget *parent;
+	DkPluginManager* pluginManager;
+	QString currRunningPlugin;
 
 	QVector<QShortcut*> shortcuts;	
 	QVector<QAction *> fileActions;
@@ -578,6 +633,8 @@ protected:
 	QVector<QAction *> panelActions;
 	QVector<QAction *> viewActions;
 	QVector<QAction *> syncActions;
+	QVector<QAction *> pluginsActions;
+	QVector<QAction *> pluginsDummyActions;
 	QVector<QAction *> lanActions;
 	QVector<QAction *> helpActions;
 	//QVector<QAction *> tcpViewerActions;
@@ -598,12 +655,13 @@ protected:
 	QMenu* panelMenu;
 	QMenu* viewMenu;
 	QMenu* syncMenu;
+	QMenu* pluginsMenu;
 	QMenu* helpMenu;
 	QMenu* contextMenu;
 
 	// sub menus
-	QMenu* fileFilesMenu;
-	QMenu* fileFoldersMenu;
+	//QMenu* fileFilesMenu;
+	//QMenu* fileFoldersMenu;
 	QMenu* panelToolsMenu;
 	DkTcpMenu* tcpViewerMenu;
 	DkTcpMenu* tcpLanMenu;
@@ -630,16 +688,22 @@ protected:
 	DkTrainDialog* trainDialog;
 	DkExplorer* explorer;
 	DkExportTiffDialog* exportTiffDialog;
-	DkMosaicDialog* mosaicDialog;
+	DkSettingsDialog* settingsDialog;
+	DkThumbsSaver* thumbSaver;
 
 	DkImageManipulationDialog* imgManipulationDialog;
 
+	DkPrintPreviewDialog* printPreviewDialog;
+	FileDownloader* fileDownloader;
+
 	DkAppManager* appManager;
 
 	// client managers
 	//DkLocalClientManager* localClientManager;
 	//DkLANClientManager* lanClientManager;
 	DkUpdater* updater;
+	DkTranslationUpdater* translationUpdater;	
+	
 
 	QRect oldGeometry;
 	QList<QToolBar *> hiddenToolbars;
@@ -663,6 +727,9 @@ protected:
 
 	virtual void readSettings();
 
+	// plugin functions
+	void addPluginsToMenu();
+	void createPluginsMenu();
 };
 
 class DllExport DkNoMacsSync : public DkNoMacs {
@@ -676,12 +743,20 @@ public:
 	
 signals:
 	void clientInitializedSignal();
+	void startRCServerSignal(bool start);
+	void startTCPServerSignal(bool start);
 
 public slots:
 	void tcpConnectAll();
+	void tcpChangeSyncMode(int syncMode, bool connectWithWhiteList = false);
+	void tcpRemoteControl(bool start);
+	void tcpRemoteDisplay(bool start);
+	void tcpAutoConnect(bool connect);
+	void startUpnpRenderer(bool start);
 	void settingsChanged();
 	void clientInitialized();
 	void newClientConnected(bool connected, bool local);
+	void startTCPServer(bool start);
 
 protected:
 
@@ -691,6 +766,7 @@ protected:
 
 	// functions
 	void initLanClient();
+	bool connectWhiteList(int mode, bool connect = true);
 
 	// gui
 	virtual void createActions();
@@ -699,6 +775,12 @@ protected:
 	// network layer
 	DkLocalManagerThread* localClient;
 	DkLanManagerThread* lanClient;
+	DkRCManagerThread* rcClient;
+#ifdef WITH_UPNP
+	QSharedPointer<DkUpnpControlPoint> upnpControlPoint;
+	QSharedPointer<DkUpnpDeviceHost> upnpDeviceHost;
+	QSharedPointer<DkUpnpRendererDeviceHost> upnpRendererDeviceHost;
+#endif // WITH_UPNP
 
 };
 
